import logging
import os

from bn import absimport, AttributeDict
from flows.framework.application import not_found
from flows.provider import start_flow, ServiceProvider
import simplejson

log = logging.getLogger(__name__)

def config(flow, name):
    return {}

def make_component(
    name, 
    required_services=None,
    available_services=None,
    enable_jsonp=False,
):
    if available_services is None:
        available_services = {}
    def run(flow):
        parts = flow.http.environ.get('PATH_INFO', '').strip('/').split('/')
        if not len(parts) == 3 or not parts[0] == 'api':
            log.error(
                'Invalid API call, %%r expected something like '
                '/%s/data/add'%(name),
                parts
            )
            return not_found(flow)
        handler_path =  os.path.join(
            flow.config.app.app_dir,
            'ext',
            'api',
            'area',
            parts[1],
            'action.py'
        )
        if not os.path.exists(handler_path):
            log.error('No api handler %r', handler_path)
            return not_found(flow)
        module_path = '%s.ext.api.area.%s.action' % (
            flow.config.app.package,
            parts[1],
        )
        mod = absimport(module_path)
        if not hasattr(mod, parts[2]):
            log.error('No api function %r', parts[2])
            return not_found(flow)
        handler = getattr(mod, parts[2])
        args = None
        if not flow.http.has_key('input'):
            flow.provider.start('http.input')
        if flow.http.input and flow.http.input.has_key('args'):
            input = flow.http.input['args'].value
            if input:
                args = simplejson.loads(input)
        flow.http.response.status = '200 OK' 
        flow.http.response.headers = [
            ('Content-Type', 'application/javascript; charset=utf-8')
        ]
        if args is None:
            result = handler(flow)
        else:
            result = handler(flow, args)
        # @@@ Should be an option.
        json = simplejson.dumps(result, ensure_ascii=False, sort_keys=True, indent=4)
        function_name = None
        if enable_jsonp:
            function_name =  flow.http.query and \
               flow.http.query.getfirst('callback') and \
               flow.http.query.getfirst('callback') or None
        if function_name:
            flow.http.response.body = ['%s(%s)'%(function_name, json)]
        else:
            flow.http.response.body = [json]

    def on_load_component(flow):
        flow.config[name] = config(flow, name)
        def dispatch(flow):
            f = AttributeDict(flow.copy())
            del f['provider']
            #if f.config.has_key('ext'):
            #    raise Exception(
            #        'Extension configuration is already present, perhaps the '
            #        '%r extension forgot to remove it?' % f.config.ext
            #    )
            #else:
            f.config['ext'] = f.config[name]
            provider = ServiceProvider(
                available_services = available_services,
                existing_services = f,
            )
            start_flow(
                run,
                provider = provider, # flow.provider.instance,
                required_services=required_services,
            )
        return dispatch
    return on_load_component

